/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#define VER         "0.1"
#define BUFFSZ      16384
#define PWDOFF      164
#define ACCOFF      19652
#define MYFREAD     ptr = buff; \
                    limit = buff + BUFFSZ; \
                    while(ptr < limit) { \
                        *ptr = fgetc(fd); \
                        if(!*ptr) break; \
                        ptr++; \
                    }
#define READSHOW(x) MYFREAD; \
                    printf(x"%s\n", buff);



typedef unsigned char   u_char;



void read_err(void);
void write_err(void);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd,
            *fdout;
    int     len;
    u_char  buff[BUFFSZ],
            *ptr,
            *limit;


    setbuf(stdout, NULL);

    fputs("\n"
        "Calypso mail mailbox file decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 3) {
        printf("\nUsage: %s <file.BOX> <output.BOX>\n"
            "\n"
            "This tool decodes the mailbox files and then shows the mailbox password if\n"
            "present and all the informations (password too) of the first account.\n"
            "If there are other accounts you must check them manually looking in the\n"
            "output file.\n"
            "\n", argv[0]);
        exit(1);
    }

    printf("Open file: %s\n", argv[1]);
    fd = fopen(argv[1], "rb");
    if(!fd) std_err();

    printf("Create file: %s\n", argv[2]);
    fdout = fopen(argv[2], "wb");
    if(!fdout) std_err();

    while(1) {
        len = fread(buff, 1, BUFFSZ, fd);
        if(!len) break;

        ptr = buff;
        limit = buff + len;
        while(ptr < limit) {
            *ptr = (*ptr + 0x43) ^ 0xff;
            ptr++;
        }

        if(fwrite(buff, len, 1, fdout) != 1) write_err();
    }

    fclose(fd);
    fclose(fdout);

    fd = fopen(argv[2], "rb");
    if(!fd) std_err();

    if(fseek(fd, PWDOFF, SEEK_SET) < 0) std_err();
    MYFREAD;
    if(*buff) printf("\nMailbox password:   %s\n", buff);

    if(fseek(fd, ACCOFF, SEEK_SET) < 0) std_err();
    fputs("\nFirst account informations:\n", stdout);
    READSHOW("Account name:    ");
    READSHOW("E-mail:          ");
    READSHOW("Real name:       ");
    READSHOW("SMTP server:     ");
    READSHOW("UserID:          ");
    READSHOW("Incoming server: ");
    READSHOW("Reply e-mail:    ");
    READSHOW("Password:        ");
    READSHOW("Audio file:      ");
    READSHOW("SMTP user:       ");
    READSHOW("SMTP password:   ");

    fclose(fd);

    printf("\n"
        "If the mailbox file contains other accounts you must check them manually, all\n"
        "the passwords are in plain-text so you need only to search for an account name\n"
        "into the file %s with a hex editor and you will find the relative password\n",
        argv[2]);

    return(0);
}



void read_err(void) {
    printf("\nError: read error, probably the mailbox file is corrupted\n");
    exit(1);
}


void write_err(void) {
    printf("\nError: write error, probably the disk is full\n");
    exit(1);
}



void std_err(void) {
    perror("\nError");
    exit(1);
}
